@@ -1,7 +1,7 @@
 /*
  * ion/ioncore/fullscreen.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2007. 
+ * Copyright (c) Tuomo Valkonen 1999-2008. 
  *
  * See the included file LICENSE for details.
  */
@@ -167,16 +167,5 @@ bool group_set_fullscreen_extl(WGroup *grp, const char *how)
 }
 
 
-/*EXTL_DOC
- * Is \var{reg} in full screen mode?
- */
-EXTL_SAFE
-EXTL_EXPORT_MEMBER
-bool group_is_fullscreen(WGroup *grp)
-{
-    return REGION_IS_FULLSCREEN(grp);
-}
-
-
 /*}}}*/
 
