@@ -1,7 +1,7 @@
 /*
  * ion/ioncore/manage.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2007. 
+ * Copyright (c) Tuomo Valkonen 1999-2008. 
  *
  * See the included file LICENSE for details.
  */
@@ -98,15 +98,21 @@ extern WPHolder *region_prepare_manage_transient_default(WRegion *reg,
 
 /* Rescue */
 
+#define REGION_RESCUE_PHFLAGS_OK 0x01
+#define REGION_RESCUE_NODEEP     0x02
+
 INTRSTRUCT(WRescueInfo);
 
+extern WPHolder *rescueinfo_pholder(WRescueInfo *info);
+
 /* if ph is given, it is used, otherwise one is looked for when needed */
-extern bool region_rescue(WRegion *reg, WPHolder *ph);
+extern bool region_rescue(WRegion *reg, WPHolder *ph, int flags);
 extern bool region_rescue_needed(WRegion *reg);
 extern bool region_rescue_clientwins(WRegion *reg, WRescueInfo *info);
 extern bool region_rescue_child_clientwins(WRegion *reg, WRescueInfo *info);
 extern bool region_rescue_some_clientwins(WRegion *reg, WRescueInfo *info,
                                           WRegionIterator *iter, void *st);
+extern bool region_do_rescue_this(WRegion *tosave, WRescueInfo *info, int ph_flags);
 
 
 #endif /* ION_IONCORE_MANAGE_H */
