@@ -1,7 +1,7 @@
 /*
  * ion/ioncore/region.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2007. 
+ * Copyright (c) Tuomo Valkonen 1999-2008. 
  *
  * See the included file LICENSE for details.
  */
@@ -283,7 +283,9 @@ bool region_prepare_focus(WRegion *reg, int flags,
                           WPrepareFocusResult *res)
 {
 
-    if(!REGION_IS_ACTIVE(reg) || !REGION_IS_MAPPED(reg)){
+    if(TRUE /* !REGION_IS_ACTIVE(reg) || 
+       !REGION_IS_MAPPED(reg) ||
+       ioncore_g.focus_next!=NULL*/){
         WRegion *mgr=REGION_MANAGER(reg);
         WRegion *par=REGION_PARENT_REG(reg);
         
@@ -494,7 +496,7 @@ bool region_dispose_(WRegion *reg, bool not_simple)
     WPHolder *ph=NULL;
     
     if(rescue){
-        if(!region_rescue(reg, NULL)){
+        if(!region_rescue(reg, NULL, 0)){
             warn(TR("Failed to rescue some client windows - not closing."));
             return FALSE;
         }
@@ -596,6 +598,9 @@ void region_unset_manager(WRegion *reg, WRegion *mgr)
     
     reg->manager=NULL;
     
+    /* Reset status, as it is set by manager */
+    reg->flags&=~REGION_SKIP_FOCUS;
+
     if(region_is_activity_r(reg))
         region_clear_mgd_activity(mgr);
     
