@@ -1,7 +1,7 @@
 /*
  * ion/mainloop/signal.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2007. 
+ * Copyright (c) Tuomo Valkonen 1999-2008. 
  *
  * See the included file LICENSE for details.
  */
@@ -54,4 +54,7 @@ extern void mainloop_trap_signals(const sigset_t *set);
 extern WHook *mainloop_sigchld_hook;
 extern WHook *mainloop_sigusr2_hook;
 
+/* Returns monotonic time if possible */
+extern int mainloop_gettime(struct timeval *val);
+
 #endif /* ION_LIBMAINLOOP_SIGNAL_H */
