@@ -14,19 +14,34 @@
 int stringintmap_ndx(const StringIntMap *map, const char *str)
 {
     int i;
-	
-	for(i=0; map[i].string!=NULL; i++){
-		if(strcmp(str, map[i].string)==0)
-			return i;
-	}
-	
-	return -1;
+    
+    for(i=0; map[i].string!=NULL; i++){
+        if(strcmp(str, map[i].string)==0)
+            return i;
+    }
+    
+    return -1;
 }
 
 
 int stringintmap_value(const StringIntMap *map, const char *str, int dflt)
 {
     int i=stringintmap_ndx(map, str);
-	return (i==-1 ? dflt : map[i].value); 
+    return (i==-1 ? dflt : map[i].value); 
 }
 
+
+const char *stringintmap_key(const StringIntMap *map, int value, 
+                             const char *dflt)
+{
+    int i;
+    
+    for(i=0; map[i].string!=NULL; ++i){
+        if(map[i].value==value){
+            return map[i].string;
+        }
+    }
+    
+    return dflt;
+    
+}
