@@ -98,7 +98,7 @@ static void put_prog_name()
     if(!progname_enable)
         return;
     
-    progname=prog_execname();
+    progname=libtu_progname();
     
     if(progname==NULL)
         return;
